(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
    typeof define === 'function' && define.amd ? define('json2html-lib', ['exports'], factory) :
    (global = global || self, factory(global['json2html-lib'] = {}));
}(this, (function (exports) { 'use strict';

    var Json2html = /** @class */ (function () {
        /**
         * @param json one ou list of node data
         * @param option formating options
         */
        function Json2html(json, option) {
            if (option === void 0) { option = {}; }
            this.json = json;
            this.options = {
                spaceType: 'space',
                spaceLength: 4,
                spaceBase: 0,
                // maxLenght: 0,
                attrPosition: 'alignFirstAttr',
                type: 'html',
                formatting: 'multiline',
                indent: true,
                xmlDefaultTag: 'span',
                noContentTags: [
                    'area',
                    'base',
                    'br',
                    'col',
                    'command',
                    'embed',
                    'hr',
                    'img',
                    'input',
                    'keygen',
                    'link',
                    'meta',
                    'param',
                    'source',
                    'track',
                    'wbr'
                ]
            };
            Object.assign(this.options, option);
        }
        /**
         * rendering in string
         */
        Json2html.prototype.toString = function () {
            var _this = this;
            var html = '';
            if (!Array.isArray(this.json)) {
                html = "" + this._getSpacing(0) + this._generate(0, this.json);
            }
            else {
                this.json.forEach(function (element, index) {
                    html += "" + (index > 0 ? '\n' : '') + _this._getSpacing(0) + _this._generate(0, element);
                });
            }
            return html;
        };
        /**
         * generation in string for a node (tag with attributes and content)
         * @param lvl node level
         * @param json node data
         * @returns render of node
         */
        Json2html.prototype._generate = function (lvl, json) {
            var hasContent = !this.options.noContentTags.includes(json.tag.toLowerCase());
            var xmlAutoClose = !hasContent && this.options.type === 'xml' ? '/' : '';
            var string = "<" + json.tag + this._generateAttrs(lvl, json) + xmlAutoClose + ">";
            if (hasContent) {
                var tagcontent = this._generateBody(lvl, json);
                if (tagcontent && this._hasMultiline()) {
                    tagcontent = tagcontent + "\n" + this._getSpacing(lvl);
                }
                string += tagcontent + "</" + json.tag + ">";
            }
            return string;
        };
        /**
         * attributes list generation
         * @param lvl level node
         * @param json node data
         * @returns attributes tag
         */
        Json2html.prototype._generateAttrs = function (lvl, json) {
            var string = '';
            var attrs = json.attrs;
            if (attrs && Object.keys(attrs).length) {
                for (var id in attrs) {
                    if (attrs[id] !== undefined) {
                        var attr = '';
                        switch (this.options.attrPosition) {
                            case 'inline':
                                attr += ' ';
                                break;
                            case 'space':
                                attr += string && this.options.indent && this._hasMultiline()
                                    ? "\n" + this._getSpacing(lvl + 1)
                                    : string += ' ';
                                break;
                            case 'alignTag':
                                attr += string && this.options.indent && this._hasMultiline()
                                    ? "\n" + this._getSpacing(lvl, 1)
                                    : ' ';
                                break;
                            case 'alignFirstAttr':
                                attr += string && this.options.indent && this._hasMultiline()
                                    ? "\n" + this._getSpacing(lvl, json.tag.length + 2)
                                    : ' ';
                                break;
                        }
                        attr += "" + id + (attrs[id] !== null || attrs[id] ? "=\"" + attrs[id] + "\"" : '');
                        string += attr;
                    }
                }
            }
            return string;
        };
        /**
         * tag body generation
         * @param lvl level node
         * @param json node data
         * @returns render of body
         */
        Json2html.prototype._generateBody = function (lvl, json) {
            var _this = this;
            var string = '';
            if (json.body) {
                if (!Array.isArray(json.body)) {
                    string += this._generateBodyElement(lvl, json.body, true);
                }
                else {
                    json.body.forEach(function (element) {
                        string += _this._generateBodyElement(lvl, element, false);
                    });
                }
            }
            return string;
        };
        /**
         * tag body generation for one node
         * @param lvl level node
         * @param element node data or string
         * @param onlyOne body this an unique node
         * @returns render of body
         */
        Json2html.prototype._generateBodyElement = function (lvl, element, onlyOne) {
            var string = '';
            if (this._hasMultiline()) {
                string += "\n" + this._getSpacing(lvl + 1);
            }
            // in XML mode, for generate a valid XML structure
            if (!onlyOne && this.options.type === 'xml' && typeof element === 'string') {
                element = {
                    tag: this.options.xmlDefaultTag,
                    body: element
                };
            }
            string += typeof element === 'string'
                ? element
                : this._generate(lvl + 1, element);
            return string;
        };
        /**
         * if multiline rendering
         */
        Json2html.prototype._hasMultiline = function () {
            return this.options.formatting === 'multiline';
        };
        /**
         * calculating the number of spaces for indentation
         * @param lvl level
         * @param addition space of space (only space, no tabulation)
         * @returns space
         */
        Json2html.prototype._getSpacing = function (lvl, addition) {
            if (addition === void 0) { addition = 0; }
            return this.options.indent
                ? (this.options.spaceType === 'space' ? ' ' : '\t')
                    .repeat((lvl + this.options.spaceBase) * this.options.spaceLength) + ' '.repeat(addition)
                : '';
        };
        return Json2html;
    }());

    /*
     * Public API Surface of json2html
     */

    /**
     * Generated bundle index. Do not edit.
     */

    exports.Json2html = Json2html;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=json2html-lib.umd.js.map
